package main

import (
	"fmt"
	"os"

	"github.com/aliyun/aliyun-oss-go-sdk/oss"
	"github.com/hyperledger/fabric-sdk-go/pkg/client/channel"
	"github.com/hyperledger/fabric-sdk-go/pkg/core/config"
	"github.com/hyperledger/fabric-sdk-go/pkg/fabsdk"
)

func main() {
	if len(os.Args) != 2 {
		fmt.Println("Usage: fetch <md5>")
		os.Exit(-100)
	}
	MD5Str := os.Args[1]

	if err := checkVaraibles(); err != nil {
		fmt.Println(err)
		os.Exit(-100)
	}

	c := config.FromFile("./connection-profile.yaml")
	sdk, err := fabsdk.New(c)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s\n", err)
		os.Exit(1)
	}
	defer sdk.Close()

	if err := enrollUser(sdk); err != nil {
		os.Exit(-1)
	}
	clientChannelContext := sdk.ChannelContext(channelName, fabsdk.WithUser(user))

	fmt.Println("======================================================")
	fmt.Println("[-] Fetch file info from chaincode...")

	client, err := channel.New(clientChannelContext)
	ret := queryCC(client, "query", [][]byte{[]byte(MD5Str)})
	if ret == "" {
		fmt.Println("[-] Not found on chaincode, exit.")
		os.Exit(-2)
	}

	ossClient, err := oss.New(endpoint, akId, akSecret)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	bucket, err := ossClient.Bucket(bucketName)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	fmt.Println("[-] Downloading to", MD5Str)
	err = bucket.GetObjectToFile(MD5Str, MD5Str)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	fmt.Println("[-] Done.")
}
