package main

import (
	"crypto/md5"
	"encoding/hex"
	"fmt"
	"io"
	"os"

	"github.com/aliyun/aliyun-oss-go-sdk/oss"
	"github.com/hyperledger/fabric-sdk-go/pkg/client/channel"
	"github.com/hyperledger/fabric-sdk-go/pkg/core/config"
	"github.com/hyperledger/fabric-sdk-go/pkg/fabsdk"
)

func main() {
	if len(os.Args) != 2 {
		fmt.Println("Usage: push <file>")
		os.Exit(-100)
	}

	if err := checkVaraibles(); err != nil {
		fmt.Println(err)
		os.Exit(-100)
	}

	c := config.FromFile("./connection-profile.yaml")
	sdk, err := fabsdk.New(c)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s\n", err)
		os.Exit(1)
	}
	defer sdk.Close()

	if err := enrollUser(sdk); err != nil {
		os.Exit(-1)
	}
	clientChannelContext := sdk.ChannelContext(channelName, fabsdk.WithUser(user))

	// calculate md5

	filePath := os.Args[1]
	file, err := os.Open(filePath)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}
	defer file.Close()

	hash := md5.New()
	if _, err := io.Copy(hash, file); err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	MD5Str := hex.EncodeToString(hash.Sum(nil))
	fmt.Println("======================================================")
	fmt.Println("[-] Uploading", filePath, "to Alibaba Cloud OSS, MD5:", MD5Str)

	ossClient, err := oss.New(endpoint, akId, akSecret)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	client, err := channel.New(clientChannelContext)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}
	bucket, err := ossClient.Bucket(bucketName)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	err = bucket.PutObjectFromFile(MD5Str, filePath)
	if err != nil {
		fmt.Println("Error:", err)
		os.Exit(-1)
	}

	fmt.Println("[-] Upload MD5 to chaincode...")
	invokeCC(client, "set", [][]byte{[]byte(MD5Str), []byte(bucketName + "." + MD5Str)})
	fmt.Println("[-] Done.")
}
