package main

import (
	"errors"
	"fmt"
	"os"

	"github.com/hyperledger/fabric-sdk-go/pkg/client/channel"
	"github.com/hyperledger/fabric-sdk-go/pkg/client/msp"
	"github.com/hyperledger/fabric-sdk-go/pkg/common/logging"
	"github.com/hyperledger/fabric-sdk-go/pkg/fabsdk"
)

var (
	lvl         = logging.ERROR
	cc          = "sacc"
	user        = os.Getenv("baas_user")
	secret      = os.Getenv("baas_secret")
	channelName = os.Getenv("baas_channel")
	akId        = os.Getenv("oss_ak")
	akSecret    = os.Getenv("oss_ak_secret")
	endpoint    = os.Getenv("oss_endpoint") // "oss-cn-hangzhou.aliyuncs.com"
	bucketName  = os.Getenv("oss_bucket")   // "baas-sdk-test1"
)

func checkVaraibles() error {
	if user == "" {
		return errors.New("environment variable 'baas_user' shoule be given")
	}

	if secret == "" {
		return errors.New("environment variable 'baas_secret' shoule be given")
	}

	if channelName == "" {
		return errors.New("environment variable 'baas_channel' shoule be given")
	}

	if akId == "" {
		return errors.New("environment variable 'oss_ak' shoule be given")
	}

	if akSecret == "" {
		return errors.New("environment variable 'oss_ak_secret' shoule be given")
	}

	if endpoint == "" {
		return errors.New("environment variable 'oss_endpoint' shoule be given")
	}

	if bucketName == "" {
		return errors.New("environment variable 'oss_bucket' shoule be given")
	}

	return nil
}

func queryCC(client *channel.Client, f string, args [][]byte) string {
	response, err := client.Query(channel.Request{
		ChaincodeID: cc,
		Fcn:         f,
		Args:        args,
	})

	if err != nil {
		fmt.Println("Failed to query: ", err)
	}

	ret := string(response.Payload)
	fmt.Println("Chaincode status: ", response.ChaincodeStatus)
	fmt.Println("Payload: ", ret)
	return ret
}

func invokeCC(client *channel.Client, f string, args [][]byte) {
	_, err := client.Execute(channel.Request{
		ChaincodeID: cc,
		Fcn:         f,
		Args:        args,
	})

	if err != nil {
		fmt.Printf("Failed to invoke: %+v\n", err)
	}
}

func enrollUser(sdk *fabsdk.FabricSDK) error {
	ctx := sdk.Context()
	mspClient, err := msp.New(ctx)
	if err != nil {
		fmt.Printf("Failed to create msp client: %s\n", err)
		return err
	}

	_, err = mspClient.GetSigningIdentity(user)
	if err == msp.ErrUserNotFound {
		fmt.Println("Going to enroll user")
		err = mspClient.Enroll(user, msp.WithSecret(secret))

		if err != nil {
			fmt.Printf("Failed to enroll user: %s\n", err)
			return err
		} else {
			fmt.Printf("Success enroll user: %s\n", user)
		}

	} else if err != nil {
		fmt.Printf("Failed to get user: %s\n", err)
		return err
	} else {
		fmt.Printf("User %s already enrolled, skip enrollment.\n", user)
	}

	return nil
}
